__license__   = 'GPL v3'
__copyright__ = '2011, Jan Beisenkamp silentguy@gmail.com'
__docformat__ = 'restructuredtext en'

import os
from calibre.devices.usbms.driver import USBMS, debug_print
from calibre.utils.localization import get_lang
from zipfile import ZipFile
from calibre.utils.config import config_dir

class OYO(USBMS):

    name           = 'Medion OYO Device Interface'
    gui_name       = 'OYO'
    description    = _('Communicate with the Medion OYO')
    author         = 'Jan Beisenkamp <silentguy@gmail.com>'
    version        = (1,2,8)
    supported_platforms = ['windows', 'osx', 'linux']
    icon = I('oyo.png');
    
    # Ordered list of supported formats
    FORMATS     = ['epub', 'html', 'pdf', 'txt']

    VENDOR_ID   = [0x0525]
    PRODUCT_ID  = [0xa4a5]
    BCD         = [0x0319]

    VENDOR_NAME = 'E-BOOK'
    WINDOWS_MAIN_MEM = WINDOWS_CARD_A_MEM = 'READER'
    if (get_lang() == 'de'):
        EXTRA_CUSTOMIZATION_MESSAGE = ['Entferne .info Datei wenn Buch entfernt wird:::Die .info Datei beinhaltet Lesezeichen, die zuletzt gelesene Seite, etc. Das Entfernen dieser Datei schafft mehr Ordnung auf dem OYO (da leere Ordner entfernt werden), bleibt sie erhalten wird sie weiter genutzt wenn das Buch erneut auf den OYO kopiert wird. Diese Option ist beim OYO2 und dem OYO 1 ab Firmware 5000 nicht mehr relevant',]
    else :
        EXTRA_CUSTOMIZATION_MESSAGE = ['Remove .info files when deleting books:::This file contains bookmarks, last read page, etc. removing leaves a cleaner device (esp. because empty folders will get removed), if you keep it this info will be available if you send the book again later on. Only relevant for OYO 1 before FW 5000',]
    EXTRA_CUSTOMIZATION_DEFAULT = [False,]
    EBOOK_DIR_MAIN = ''
    SUPPORTS_SUB_DIRS = True
    THUMBNAIL_HEIGHT = 200

    def initialize(self):
        USBMS.initialize(self)
        imagepath=os.path.join(config_dir,'resources','images')
        if not os.path.exists(imagepath):
            os.makedirs(imagepath)
        with ZipFile(self.plugin_path, 'r') as zf:
            with open(os.path.join(imagepath,'oyo.png'),'wb') as f:
                f.write(zf.read('oyo.png'))

            
            
    def upload_cover(self, path, filename, metadata, filepath):
        coverdata = getattr(metadata, 'thumbnail', None)
        if coverdata and coverdata[2]:
            if not os.path.exists(os.path.join(path,'.info')):
                os.mkdir(os.path.join(path,'.info'))
            try:
                coverpath = '%s.jpg' % os.path.join(path,'.info', filename+os.path.splitext(filepath)[1])
                if os.path.exists(coverpath):
                    # Delete old Cover
                    debug_print('OYO: Removing old cover')
                    os.unlink(coverpath)
                coverfile = open(coverpath, 'wb')
                coverpath2 = '%s.jpg' % os.path.join(path, filename+os.path.splitext(filepath)[1])
                if os.path.exists(coverpath2):
                    # Delete old Cover
                    debug_print('OYO: Removing old cover')
                    os.unlink(coverpath2)
                coverfile2 = open(coverpath2, 'wb')
                debug_print('OYO: Sending cover to '+coverpath+' and '+coverpath2)
                coverfile.write(coverdata[2])
                coverfile2.write(coverdata[2])
            except IOError as (errno, strerror):
                debug_print("OYO: I/O error({0}): {1}".format(errno, strerror))
            finally:
                coverfile.close
                coverfile2.close

    def delete_books(self, paths, end_session=True):
        debug_print('OYO: deleting %d books'%(len(paths)))
        opts = self.settings()
        if opts.extra_customization: 
            if isinstance(opts.extra_customization,basestring):
                debug_print('OYO: Old plugin config detected. It\'s recommended to delete '+os.path.join(config_dir,'device_drivers_OYO.py'))
                deleteinfo = (opts.extra_customization.lower().strip()=='yes')
            else:
                deleteinfo = opts.extra_customization[0]
        else:
            deleteinfo=False
        for i, path in enumerate(paths):
            self.report_progress((i+1) / float(len(paths)), _('Removing books from device...'))
            path = self.normalize_path(path)
            folder = os.path.split(path)[0]
            file = os.path.split(path)[1]
            if os.path.exists(path):
                # Delete the ebook
                os.unlink(path)
                debug_print('OYO: Removed '+path)
            if os.path.exists(os.path.join(folder,'.info',file+'.jpg')):
                # Delete the cover
                os.unlink(os.path.join(folder,'.info',file+'.jpg'))
                debug_print('OYO: Removed '+os.path.join(folder,'.info',file+'.jpg'))
                os.unlink(os.path.join(folder,file+'.jpg'))
                debug_print('OYO: Removed '+os.path.join(folder,file+'.jpg'))
            if os.path.exists(os.path.join(folder,'.info',file+'.info')) and deleteinfo :
                # Delete info file
                os.unlink(os.path.join(folder,'.info',file+'.info'))
                debug_print('OYO: Removed '+os.path.join(folder,'.info',file+'.info'))
            try:
                # Remove .info folder and all parents if possible
                os.removedirs(os.path.join(folder,'.info'))
                debug_print('OYO: Removed '+os.path.join(folder,'.info'))
            except:
                # .info folder was not empty, ignore
                pass

        self.report_progress(1.0, _('Removing books from device...'))
        debug_print('OYO: finished deleting %d books'%(len(paths)))



